@@ -1,14 +1,14 @@
 #!/bin/sh
 
 if [ -z "$LDBDIR" ]; then
-    LDBDIR=`dirname $0`/..
-    export LDBDIR
+	LDBDIR=$(dirname $0)/..
+	export LDBDIR
 fi
 
 mkdir -p $LDBDIR/tests/tmp/db
 
 # running slapd in the background (with &) means it stays in the same process group, so it can be
 # killed by timelimit
-slapd -d0 -f $LDBDIR/tests/slapd.conf -h "`$LDBDIR/tests/ldapi_url.sh`" $* &
+slapd -d0 -f $LDBDIR/tests/slapd.conf -h "$($LDBDIR/tests/ldapi_url.sh)" $* &
 
 sleep 2
