@@ -47,7 +47,8 @@ sub Template($)
 
 #include \"includes.h\"
 #include \"ntdomain.h\"
-#include \"../librpc/gen_ndr/srv_$name.h\"
+#include \"librpc/gen_ndr/ndr_$name.h\"
+#include \"librpc/gen_ndr/ndr_$name\_scompat.h\"
 
 ";
 
@@ -73,6 +74,8 @@ $pad"."struct $fname *r)
 		$res .= "\treturn WERR_NOT_SUPPORTED;\n";
 	} elsif ($d->{RETURN_TYPE} eq "HRESULT") {
 		$res .= "\treturn HRES_ERROR_NOT_SUPPORTED;\n";
+	} elsif ($d->{RETURN_TYPE} eq "int") {
+		$res .= "\treturn ENOTSUP;\n";
 	}
 
 	$res .= "}
@@ -80,6 +83,10 @@ $pad"."struct $fname *r)
 ";
 		}
 	}
+$res .=
+"/* include the generated boilerplate */
+#include \"librpc/gen_ndr/ndr_$name\_scompat.c\"\
+";
 }
 
 
