@@ -26,6 +26,7 @@
 #define __LIBSMBCLIENT_INTERNAL__
 #include "libsmbclient.h"
 #include "libsmb_internal.h"
+#include "lib/param/param.h"
 
 
 /** Get the netbios name used for making connections */
@@ -94,7 +95,7 @@ smbc_setDebug(SMBCCTX *c, int debug)
 	TALLOC_CTX *frame = talloc_stackframe();
 	snprintf(buf, sizeof(buf), "%d", debug);
         c->debug = debug;
-	lp_set_cmdline("log level", buf);
+	lpcfg_set_cmdline(c->internal->lp_ctx, "log level", buf);
 	TALLOC_FREE(frame);
 }
 
@@ -189,7 +190,7 @@ smbc_setOptionDebugToStderr(SMBCCTX *c, smbc_bool b)
 		/*
 		 * We do not have a unique per-thread debug state? For
 		 * now, we'll just leave it up to the user. If any one
-		 * context spefies debug to stderr then all will be (and
+		 * context specifies debug to stderr then all will be (and
 		 * will stay that way, as it is unsafe to flip back if
 		 * stdout is in use for other things)
 		 */
@@ -534,16 +535,34 @@ smbc_setOptionProtocols(SMBCCTX *c,
 	bool ok = true;
 
 	if (min_proto != NULL) {
-		ok = lp_set_cmdline("client min protocol", min_proto);
+		ok = lpcfg_set_cmdline(c->internal->lp_ctx,
+				       "client min protocol",
+				       min_proto);
 	}
 
 	if (max_proto != NULL) {
-		ok &= lp_set_cmdline("client max protocol", max_proto);
+		ok &= lpcfg_set_cmdline(c->internal->lp_ctx,
+					"client max protocol",
+					max_proto);
 	}
 
 	return ok;
 }
 
+/** Get whether to enable POSIX extensions if available */
+smbc_bool
+smbc_getOptionPosixExtensions(SMBCCTX *c)
+{
+	return c->internal->posix_extensions;
+}
+
+/** Set whether to enable POSIX extensions if available */
+void
+smbc_setOptionPosixExtensions(SMBCCTX *c, smbc_bool b)
+{
+	c->internal->posix_extensions = b;
+}
+
 /** Get the function for obtaining authentication data */
 smbc_get_auth_data_fn
 smbc_getFunctionAuthData(SMBCCTX *c)
@@ -907,6 +926,16 @@ void smbc_setFunctionReaddirPlus(SMBCCTX *c, smbc_readdirplus_fn fn)
 	c->readdirplus = fn;
 }
 
+smbc_readdirplus2_fn smbc_getFunctionReaddirPlus2(SMBCCTX *c)
+{
+	return c->readdirplus2;
+}
+
+void smbc_setFunctionReaddirPlus2(SMBCCTX *c, smbc_readdirplus2_fn fn)
+{
+	c->readdirplus2 = fn;
+}
+
 smbc_getdents_fn
 smbc_getFunctionGetdents(SMBCCTX *c)
 {
