@@ -24,6 +24,7 @@
 #include "../libcli/smb/smb_common.h"
 #include "../lib/util/tevent_ntstatus.h"
 #include "rpc_server/srv_pipe_hnd.h"
+#include "libcli/security/security.h"
 
 #undef DBGC_CLASS
 #define DBGC_CLASS DBGC_SMB2
@@ -33,7 +34,7 @@ static struct tevent_req *smbd_smb2_write_send(TALLOC_CTX *mem_ctx,
 					       struct smbd_smb2_request *smb2req,
 					       struct files_struct *in_fsp,
 					       DATA_BLOB in_data,
-					       uint64_t in_offset,
+					       off_t in_offset,
 					       uint32_t in_flags);
 static NTSTATUS smbd_smb2_write_recv(struct tevent_req *req,
 				     uint32_t *out_count);
@@ -47,7 +48,7 @@ NTSTATUS smbd_smb2_request_process_write(struct smbd_smb2_request *req)
 	uint16_t in_data_offset;
 	uint32_t in_data_length;
 	DATA_BLOB in_data_buffer;
-	uint64_t in_offset;
+	off_t in_offset;
 	uint64_t in_file_id_persistent;
 	uint64_t in_file_id_volatile;
 	struct files_struct *in_fsp;
@@ -64,7 +65,7 @@ NTSTATUS smbd_smb2_request_process_write(struct smbd_smb2_request *req)
 
 	in_data_offset		= SVAL(inbody, 0x02);
 	in_data_length		= IVAL(inbody, 0x04);
-	in_offset		= BVAL(inbody, 0x08);
+	in_offset		= PULL_LE_I64(inbody, 0x08);
 	in_file_id_persistent	= BVAL(inbody, 0x10);
 	in_file_id_volatile	= BVAL(inbody, 0x18);
 	in_flags		= IVAL(inbody, 0x2C);
@@ -177,7 +178,7 @@ struct smbd_smb2_write_state {
 	files_struct *fsp;
 	bool write_through;
 	uint32_t in_length;
-	uint64_t in_offset;
+	off_t in_offset;
 	uint32_t out_count;
 };
 
@@ -265,7 +266,7 @@ static struct tevent_req *smbd_smb2_write_send(TALLOC_CTX *mem_ctx,
 					       struct smbd_smb2_request *smb2req,
 					       struct files_struct *fsp,
 					       DATA_BLOB in_data,
-					       uint64_t in_offset,
+					       off_t in_offset,
 					       uint32_t in_flags)
 {
 	NTSTATUS status;
@@ -297,7 +298,7 @@ static struct tevent_req *smbd_smb2_write_send(TALLOC_CTX *mem_ctx,
 	DEBUG(10,("smbd_smb2_write: %s - %s\n",
 		  fsp_str_dbg(fsp), fsp_fnum_dbg(fsp)));
 
-	smbreq = smbd_smb2_fake_smb_request(smb2req);
+	smbreq = smbd_smb2_fake_smb_request(smb2req, fsp);
 	if (tevent_req_nomem(smbreq, req)) {
 		return tevent_req_post(req, ev);
 	}
@@ -307,6 +308,7 @@ static struct tevent_req *smbd_smb2_write_send(TALLOC_CTX *mem_ctx,
 
 	if (IS_IPC(smbreq->conn)) {
 		struct tevent_req *subreq = NULL;
+                bool ok;
 
 		if (!fsp_is_np(fsp)) {
 			tevent_req_nterror(req, NT_STATUS_FILE_CLOSED);
@@ -323,11 +325,36 @@ static struct tevent_req *smbd_smb2_write_send(TALLOC_CTX *mem_ctx,
 		tevent_req_set_callback(subreq,
 					smbd_smb2_write_pipe_done,
 					req);
+
+		/*
+		 * Make sure we mark the fsp as having outstanding async
+		 * activity so we don't crash on shutdown close.
+		 */
+
+		ok = aio_add_req_to_fsp(fsp, req);
+		if (!ok) {
+			tevent_req_nterror(req, NT_STATUS_NO_MEMORY);
+			return tevent_req_post(req, ev);
+		}
+
 		return req;
 	}
 
-	if (!CHECK_WRITE(fsp)) {
-		tevent_req_nterror(req, NT_STATUS_ACCESS_DENIED);
+	status = check_any_access_fsp(fsp, FILE_WRITE_DATA|FILE_APPEND_DATA);
+	if (!NT_STATUS_IS_OK(status)) {
+		tevent_req_nterror(req, status);
+		return tevent_req_post(req, ev);
+	}
+
+	if (state->in_offset == VFS_PWRITE_APPEND_OFFSET &&
+	    !fsp->fsp_flags.posix_append)
+	{
+		tevent_req_nterror(req, NT_STATUS_INVALID_PARAMETER);
+		return tevent_req_post(req, ev);
+	} else if (fsp->fsp_flags.posix_append &&
+		   state->in_offset != VFS_PWRITE_APPEND_OFFSET)
+	{
+		tevent_req_nterror(req, NT_STATUS_INVALID_PARAMETER);
 		return tevent_req_post(req, ev);
 	}
 
