@@ -39,6 +39,7 @@
 #include "lib/util/tevent_ntstatus.h"
 #include "lib/util/iov_buf.h"
 #include "lib/global_contexts.h"
+#include "source3/include/util_tdb.h"
 
 struct smbXsrv_client_table {
 	struct {
@@ -54,7 +55,7 @@ static struct db_context *smbXsrv_client_global_db_ctx = NULL;
 
 NTSTATUS smbXsrv_client_global_init(void)
 {
-	const char *global_path = NULL;
+	char *global_path = NULL;
 	struct db_context *backend = NULL;
 	struct db_context *db_ctx = NULL;
 
@@ -78,11 +79,9 @@ NTSTATUS smbXsrv_client_global_init(void)
 			  O_RDWR | O_CREAT, 0600,
 			  DBWRAP_LOCK_ORDER_1,
 			  DBWRAP_FLAG_NONE);
+	TALLOC_FREE(global_path);
 	if (backend == NULL) {
-		NTSTATUS status;
-
-		status = map_nt_error_from_unix_common(errno);
-
+		NTSTATUS status = map_nt_error_from_unix_common(errno);
 		return status;
 	}
 
@@ -97,28 +96,15 @@ NTSTATUS smbXsrv_client_global_init(void)
 	return NT_STATUS_OK;
 }
 
-/*
- * NOTE:
- * We need to store the keys in big endian so that dbwrap_rbt's memcmp
- * has the same result as integer comparison between the uint32_t
- * values.
- *
- * TODO: implement string based key
- */
-
 #define SMBXSRV_CLIENT_GLOBAL_TDB_KEY_SIZE 16
 
 static TDB_DATA smbXsrv_client_global_id_to_key(const struct GUID *client_guid,
 						uint8_t *key_buf)
 {
 	TDB_DATA key = { .dsize = 0, };
-	NTSTATUS status;
 	struct GUID_ndr_buf buf = { .buf = {0}, };
 
-	status = GUID_to_ndr_buf(client_guid, &buf);
-	if (!NT_STATUS_IS_OK(status)) {
-		return key;
-	}
+	GUID_to_ndr_buf(client_guid, &buf);
 	memcpy(key_buf, buf.buf, SMBXSRV_CLIENT_GLOBAL_TDB_KEY_SIZE);
 
 	key = make_tdb_data(key_buf, SMBXSRV_CLIENT_GLOBAL_TDB_KEY_SIZE);
@@ -143,7 +129,7 @@ static struct db_record *smbXsrv_client_global_fetch_locked(
 		struct GUID_txt_buf buf;
 		DBG_DEBUG("Failed to lock guid [%s], key '%s'\n",
 			  GUID_buf_string(client_guid, &buf),
-			  hex_encode_talloc(talloc_tos(), key.dptr, key.dsize));
+			  tdb_data_dbg(key));
 	}
 
 	return rec;
@@ -234,7 +220,7 @@ static void smbXsrv_client_global_verify_record(struct db_record *db_rec,
 	if (!NDR_ERR_CODE_IS_SUCCESS(ndr_err)) {
 		NTSTATUS status = ndr_map_error2ntstatus(ndr_err);
 		DBG_WARNING("key '%s' ndr_pull_struct_blob - %s\n",
-			    hex_encode_talloc(frame, key.dptr, key.dsize),
+			    tdb_data_dbg(key),
 			    nt_errstr(status));
 		TALLOC_FREE(frame);
 		return;
@@ -246,8 +232,8 @@ static void smbXsrv_client_global_verify_record(struct db_record *db_rec,
 	}
 
 	if (global_blob.version != SMBXSRV_VERSION_0) {
-		DBG_ERR("key '%s' use unsupported version %u\n",
-			hex_encode_talloc(frame, key.dptr, key.dsize),
+		DBG_ERR("key '%s' uses unsupported version %u\n",
+			tdb_data_dbg(key),
 			global_blob.version);
 		NDR_PRINT_DEBUG(smbXsrv_client_globalB, &global_blob);
 		TALLOC_FREE(frame);
@@ -261,7 +247,7 @@ static void smbXsrv_client_global_verify_record(struct db_record *db_rec,
 		struct server_id_buf tmp;
 
 		DBG_NOTICE("key '%s' server_id %s is already dead.\n",
-			   hex_encode_talloc(frame, key.dptr, key.dsize),
+			   tdb_data_dbg(key),
 			   server_id_str_buf(global->server_id, &tmp));
 		if (DEBUGLVL(DBGLVL_NOTICE)) {
 			NDR_PRINT_DEBUG(smbXsrv_client_globalB, &global_blob);
@@ -277,7 +263,7 @@ static void smbXsrv_client_global_verify_record(struct db_record *db_rec,
 		struct server_id_buf tmp;
 
 		DBG_NOTICE("key '%s' server_id %s does not exist.\n",
-			   hex_encode_talloc(frame, key.dptr, key.dsize),
+			   tdb_data_dbg(key),
 			   server_id_str_buf(global->server_id, &tmp));
 		if (DEBUGLVL(DBGLVL_NOTICE)) {
 			NDR_PRINT_DEBUG(smbXsrv_client_globalB, &global_blob);
@@ -439,13 +425,14 @@ static NTSTATUS smbXsrv_client_global_store(struct smbXsrv_client_global0 *globa
 	key = dbwrap_record_get_key(global->db_rec);
 	val = dbwrap_record_get_value(global->db_rec);
 
-	ZERO_STRUCT(global_blob);
-	global_blob.version = smbXsrv_version_global_current();
+	global_blob = (struct smbXsrv_client_globalB) {
+		.version = smbXsrv_version_global_current(),
+		.info.info0 = global,
+	};
 	if (val.dsize >= 8) {
 		global_blob.seqnum = IVAL(val.dptr, 4);
 	}
 	global_blob.seqnum += 1;
-	global_blob.info.info0 = global;
 
 	global->stored = true;
 	ndr_err = ndr_push_struct_blob(&blob, global->db_rec, &global_blob,
@@ -454,7 +441,7 @@ static NTSTATUS smbXsrv_client_global_store(struct smbXsrv_client_global0 *globa
 	if (!NDR_ERR_CODE_IS_SUCCESS(ndr_err)) {
 		status = ndr_map_error2ntstatus(ndr_err);
 		DBG_WARNING("key '%s' ndr_push - %s\n",
-			hex_encode_talloc(global->db_rec, key.dptr, key.dsize),
+			tdb_data_dbg(key),
 			nt_errstr(status));
 		TALLOC_FREE(global->db_rec);
 		return status;
@@ -464,7 +451,7 @@ static NTSTATUS smbXsrv_client_global_store(struct smbXsrv_client_global0 *globa
 	status = dbwrap_record_store(global->db_rec, val, TDB_REPLACE);
 	if (!NT_STATUS_IS_OK(status)) {
 		DBG_WARNING("key '%s' store - %s\n",
-			hex_encode_talloc(global->db_rec, key.dptr, key.dsize),
+			tdb_data_dbg(key),
 			nt_errstr(status));
 		TALLOC_FREE(global->db_rec);
 		return status;
@@ -474,7 +461,7 @@ static NTSTATUS smbXsrv_client_global_store(struct smbXsrv_client_global0 *globa
 
 	if (DEBUGLVL(DBGLVL_DEBUG)) {
 		DBG_DEBUG("key '%s' stored\n",
-			hex_encode_talloc(global->db_rec, key.dptr, key.dsize));
+			  tdb_data_dbg(key));
 		NDR_PRINT_DEBUG(smbXsrv_client_globalB, &global_blob);
 	}
 
@@ -905,14 +892,13 @@ static NTSTATUS smbXsrv_client_global_remove(struct smbXsrv_client_global0 *glob
 	status = dbwrap_record_delete(global->db_rec);
 	if (!NT_STATUS_IS_OK(status)) {
 		DBG_WARNING("key '%s' delete - %s\n",
-			hex_encode_talloc(global->db_rec, key.dptr, key.dsize),
+			tdb_data_dbg(key),
 			nt_errstr(status));
 		TALLOC_FREE(global->db_rec);
 		return status;
 	}
 	global->stored = false;
-	DBG_DEBUG("key '%s' delete\n",
-		  hex_encode_talloc(global->db_rec, key.dptr, key.dsize));
+	DBG_DEBUG("key '%s' delete\n", tdb_data_dbg(key));
 
 	TALLOC_FREE(global->db_rec);
 
@@ -1336,7 +1322,7 @@ static void smbXsrv_client_connection_drop_loop(struct tevent_req *subreq)
 	{
 		struct GUID_txt_buf buf1, buf2;
 
-		DBG_WARNING("client's client_guid [%s] != droped guid [%s]\n",
+		DBG_WARNING("client's client_guid [%s] != dropped guid [%s]\n",
 			    GUID_buf_string(&client->global->client_guid,
 					    &buf1),
 			    GUID_buf_string(&drop_info0->client_guid,
@@ -1351,7 +1337,7 @@ static void smbXsrv_client_connection_drop_loop(struct tevent_req *subreq)
 	    drop_info0->client_connect_time)
 	{
 		DBG_WARNING("client's initial connect time [%s] (%llu) != "
-			"droped initial connect time [%s] (%llu)\n",
+			"dropped initial connect time [%s] (%llu)\n",
 			nt_time_string(talloc_tos(),
 				       client->global->initial_connect_time),
 			(unsigned long long)client->global->initial_connect_time,
@@ -1449,7 +1435,7 @@ NTSTATUS smbXsrv_client_remove(struct smbXsrv_client *client)
 		};
 		struct GUID_txt_buf buf;
 
-		DBG_DEBUG("client_guid[%s] stored\n",
+		DBG_DEBUG("client_guid[%s] removed\n",
 			  GUID_buf_string(&client->global->client_guid, &buf));
 		NDR_PRINT_DEBUG(smbXsrv_clientB, &client_blob);
 	}
