@@ -35,6 +35,7 @@ struct dcesrv_lsa_TranslatedItem {
 	uint32_t flags;
 	uint32_t wb_idx;
 	bool done;
+	bool invalid_sid;
 	struct {
 		const char *domain; /* only $DOMAIN\ */
 		const char *namespace; /* $NAMESPACE\ or @$NAMESPACE */
@@ -380,6 +381,10 @@ static NTSTATUS dcesrv_lsa_LookupSids_base_call(struct dcesrv_lsa_LookupSids_bas
 			status = view->lookup_sid(state, item);
 			if (NT_STATUS_IS_OK(status)) {
 				item->done = true;
+			} else if (NT_STATUS_EQUAL(status, NT_STATUS_INVALID_SID)) {
+				item->done = true;
+				item->invalid_sid = true;
+				status = NT_STATUS_OK;
 			} else if (NT_STATUS_EQUAL(status, NT_STATUS_NONE_MAPPED)) {
 				status = NT_STATUS_OK;
 			} else if (NT_STATUS_EQUAL(status, NT_STATUS_SOME_NOT_MAPPED)) {
@@ -438,6 +443,7 @@ static NTSTATUS dcesrv_lsa_LookupSids_base_finish(
 	struct dcesrv_lsa_LookupSids_base_state *state)
 {
 	struct lsa_LookupSids3 *r = &state->r;
+	uint32_t num_invalid_sid = 0;
 	uint32_t i;
 
 	for (i=0;i<r->in.sids->num_sids;i++) {
@@ -470,9 +476,18 @@ static NTSTATUS dcesrv_lsa_LookupSids_base_finish(
 		if (item->type != SID_NAME_UNKNOWN) {
 			(*r->out.count)++;
 		}
+		if (item->invalid_sid) {
+			num_invalid_sid++;
+		}
 	}
 
 	if (*r->out.count == 0) {
+		if (num_invalid_sid != 0) {
+			for (i=0;i<r->out.names->count;i++) {
+				r->out.names->names[i].name.string = NULL;
+			}
+			return NT_STATUS_INVALID_SID;
+		}
 		return NT_STATUS_NONE_MAPPED;
 	}
 	if (*r->out.count != r->in.sids->num_sids) {
@@ -600,10 +615,7 @@ static void dcesrv_lsa_LookupSids_base_done(struct tevent_req *subreq)
 	state->r.out.result = status;
 	dcesrv_lsa_LookupSids_base_map(state);
 
-	status = dcesrv_reply(dce_call);
-	if (!NT_STATUS_IS_OK(status)) {
-		DEBUG(0,(__location__ ": dcesrv_reply() failed - %s\n", nt_errstr(status)));
-	}
+	dcesrv_async_reply(dce_call);
 }
 
 /*
@@ -663,36 +675,97 @@ NTSTATUS dcesrv_lsa_LookupSids2(struct dcesrv_call_state *dce_call,
 	return status;
 }
 
+/* A random hexadecimal number (honest!) */
+#define LSA_SERVER_IMPLICIT_POLICY_STATE_MAGIC 0xc0c99e00
 
 /*
-  lsa_LookupSids3
-
-  Identical to LookupSids2, but doesn't take a policy handle
-  
+  Ensure we're operating on an schannel connection,
+  and use a lsa_policy_state cache on the connection.
 */
-NTSTATUS dcesrv_lsa_LookupSids3(struct dcesrv_call_state *dce_call,
-				TALLOC_CTX *mem_ctx,
-				struct lsa_LookupSids3 *r)
+static NTSTATUS schannel_call_setup(struct dcesrv_call_state *dce_call,
+				    struct lsa_policy_state **_policy_state)
 {
+	struct lsa_policy_state *policy_state = NULL;
 	enum dcerpc_transport_t transport =
 		dcerpc_binding_get_transport(dce_call->conn->endpoint->ep_description);
-	const struct dcesrv_auth *auth = &dce_call->conn->auth_state;
-	struct dcesrv_lsa_LookupSids_base_state *state = NULL;
-	NTSTATUS status;
+	enum dcerpc_AuthType auth_type = DCERPC_AUTH_TYPE_NONE;
+	enum dcerpc_AuthLevel auth_level = DCERPC_AUTH_LEVEL_NONE;
 
 	if (transport != NCACN_IP_TCP) {
-		DCESRV_FAULT(DCERPC_FAULT_ACCESS_DENIED);
+		/* We can't call DCESRV_FAULT() in the sub-function */
+		dce_call->fault_code = DCERPC_FAULT_ACCESS_DENIED;
+		return NT_STATUS_ACCESS_DENIED;
 	}
 
 	/*
 	 * We don't have policy handles on this call. So this must be restricted
 	 * to crypto connections only.
+	 *
+	 * NB. gensec requires schannel connections to
+	 * have at least DCERPC_AUTH_LEVEL_INTEGRITY.
 	 */
-	if (auth->auth_type != DCERPC_AUTH_TYPE_SCHANNEL ||
-	    auth->auth_level < DCERPC_AUTH_LEVEL_INTEGRITY) {
-		DCESRV_FAULT(DCERPC_FAULT_ACCESS_DENIED);
+	dcesrv_call_auth_info(dce_call, &auth_type, &auth_level);
+	if (auth_type == DCERPC_AUTH_TYPE_KRB5 &&
+	    auth_level == DCERPC_AUTH_LEVEL_PRIVACY)
+	{
+		/* ok */
+	} else if (auth_type == DCERPC_AUTH_TYPE_SCHANNEL) {
+		/* ok - implies at least DCERPC_AUTH_LEVEL_INTEGRITY. */
+	} else {
+		/* We can't call DCESRV_FAULT() in the sub-function */
+		dce_call->fault_code = DCERPC_FAULT_ACCESS_DENIED;
+		return NT_STATUS_ACCESS_DENIED;
 	}
 
+	/*
+	 * We don't have a policy handle on this call, so we want to
+	 * make a policy state and cache it for the life of the
+	 * connection, to avoid re-opening the DB each call.
+	 */
+	policy_state
+		= dcesrv_iface_state_find_conn(dce_call,
+					       LSA_SERVER_IMPLICIT_POLICY_STATE_MAGIC,
+					       struct lsa_policy_state);
+
+	if (policy_state == NULL) {
+		NTSTATUS status
+			= dcesrv_lsa_get_policy_state(dce_call,
+						      dce_call /* mem_ctx */,
+						      0, /* we skip access checks */
+						      &policy_state);
+		if (!NT_STATUS_IS_OK(status)) {
+			return status;
+		}
+
+		/*
+		 * This will talloc_steal() policy_state onto the
+		 * connection, which has longer lifetime than the
+		 * immediate caller requires
+		 */
+		status = dcesrv_iface_state_store_conn(dce_call,
+						       LSA_SERVER_IMPLICIT_POLICY_STATE_MAGIC,
+						       policy_state);
+		if (!NT_STATUS_IS_OK(status)) {
+			return status;
+		}
+	}
+	*_policy_state = policy_state;
+	return NT_STATUS_OK;
+}
+
+/*
+  lsa_LookupSids3
+
+  Identical to LookupSids2, but doesn't take a policy handle
+
+*/
+NTSTATUS dcesrv_lsa_LookupSids3(struct dcesrv_call_state *dce_call,
+				TALLOC_CTX *mem_ctx,
+				struct lsa_LookupSids3 *r)
+{
+	struct dcesrv_lsa_LookupSids_base_state *state = NULL;
+	NTSTATUS status;
+
 	*r->out.domains = NULL;
 	r->out.names->count = 0;
 	r->out.names->names = NULL;
@@ -703,16 +776,27 @@ NTSTATUS dcesrv_lsa_LookupSids3(struct dcesrv_call_state *dce_call,
 		return NT_STATUS_NO_MEMORY;
 	}
 
-	state->dce_call = dce_call;
-	state->mem_ctx = mem_ctx;
-
-	status = dcesrv_lsa_get_policy_state(state->dce_call, mem_ctx,
-					     0, /* we skip access checks */
-					     &state->policy_state);
+	/*
+	 * We don't have a policy handle on this call, so we want to
+	 * make a policy state and cache it for the life of the
+	 * connection, to avoid re-opening the DB each call.
+	 *
+	 * This also enforces that this is only available over
+	 * ncacn_ip_tcp and with SCHANNEL.
+	 *
+	 * schannel_call_setup may also set the fault state.
+	 *
+	 * state->policy_state is shared between all calls on this
+	 * connection and is moved with talloc_steal() under the
+	 * connection, not dce_call or state.
+	 */
+	status = schannel_call_setup(dce_call, &state->policy_state);
 	if (!NT_STATUS_IS_OK(status)) {
 		return status;
 	}
 
+	state->dce_call = dce_call;
+	state->mem_ctx = mem_ctx;
 	state->r.in.sids = r->in.sids;
 	state->r.in.level = r->in.level;
 	state->r.in.lookup_options = r->in.lookup_options;
@@ -1220,10 +1304,7 @@ static void dcesrv_lsa_LookupNames_base_done(struct tevent_req *subreq)
 	state->r.out.result = status;
 	dcesrv_lsa_LookupNames_base_map(state);
 
-	status = dcesrv_reply(dce_call);
-	if (!NT_STATUS_IS_OK(status)) {
-		DEBUG(0,(__location__ ": dcesrv_reply() failed - %s\n", nt_errstr(status)));
-	}
+	dcesrv_async_reply(dce_call);
 }
 
 /*
@@ -1293,25 +1374,9 @@ NTSTATUS dcesrv_lsa_LookupNames3(struct dcesrv_call_state *dce_call,
 NTSTATUS dcesrv_lsa_LookupNames4(struct dcesrv_call_state *dce_call, TALLOC_CTX *mem_ctx,
 				 struct lsa_LookupNames4 *r)
 {
-	enum dcerpc_transport_t transport =
-		dcerpc_binding_get_transport(dce_call->conn->endpoint->ep_description);
-	const struct dcesrv_auth *auth = &dce_call->conn->auth_state;
 	struct dcesrv_lsa_LookupNames_base_state *state = NULL;
 	NTSTATUS status;
 
-	if (transport != NCACN_IP_TCP) {
-		DCESRV_FAULT(DCERPC_FAULT_ACCESS_DENIED);
-	}
-
-	/*
-	 * We don't have policy handles on this call. So this must be restricted
-	 * to crypto connections only.
-	 */
-	if (auth->auth_type != DCERPC_AUTH_TYPE_SCHANNEL ||
-	    auth->auth_level < DCERPC_AUTH_LEVEL_INTEGRITY) {
-		DCESRV_FAULT(DCERPC_FAULT_ACCESS_DENIED);
-	}
-
 	*r->out.domains = NULL;
 	r->out.sids->count = 0;
 	r->out.sids->sids = NULL;
@@ -1325,9 +1390,21 @@ NTSTATUS dcesrv_lsa_LookupNames4(struct dcesrv_call_state *dce_call, TALLOC_CTX
 	state->dce_call = dce_call;
 	state->mem_ctx = mem_ctx;
 
-	status = dcesrv_lsa_get_policy_state(state->dce_call, state,
-					     0, /* we skip access checks */
-					     &state->policy_state);
+	/*
+	 * We don't have a policy handle on this call, so we want to
+	 * make a policy state and cache it for the life of the
+	 * connection, to avoid re-opening the DB each call.
+	 *
+	 * This also enforces that this is only available over
+	 * ncacn_ip_tcp and with SCHANNEL.
+	 *
+	 * schannel_call_setup may also set the fault state.
+	 *
+	 * state->policy_state is shared between all calls on this
+	 * connection and is moved with talloc_steal() under the
+	 * connection, not dce_call or state.
+	 */
+	status = schannel_call_setup(dce_call, &state->policy_state);
 	if (!NT_STATUS_IS_OK(status)) {
 		return status;
 	}
@@ -1595,7 +1672,7 @@ static NTSTATUS dcesrv_lsa_lookup_name_builtin(
 			return NT_STATUS_NONE_MAPPED;
 		}
 		/*
-		 * We know we're authoritive
+		 * We know we're authoritative
 		 */
 		status = NT_STATUS_OK;
 	}
@@ -1636,7 +1713,7 @@ static NTSTATUS dcesrv_lsa_lookup_sid_builtin(
 				       &item->type);
 	if (NT_STATUS_EQUAL(status, NT_STATUS_NONE_MAPPED)) {
 		/*
-		 * We know we're authoritive
+		 * We know we're authoritative
 		 */
 		status = NT_STATUS_OK;
 	}
@@ -1802,7 +1879,7 @@ static NTSTATUS dcesrv_lsa_lookup_name_account(
 			return NT_STATUS_NONE_MAPPED;
 		}
 		/*
-		 * We know we're authoritive
+		 * We know we're authoritative
 		 */
 		status = NT_STATUS_OK;
 	}
@@ -1846,7 +1923,7 @@ static NTSTATUS dcesrv_lsa_lookup_sid_account(
 				       &item->type);
 	if (NT_STATUS_EQUAL(status, NT_STATUS_NONE_MAPPED)) {
 		/*
-		 * We know we're authoritive
+		 * We know we're authoritative
 		 */
 		status = NT_STATUS_OK;
 	}
@@ -1875,6 +1952,8 @@ static NTSTATUS dcesrv_lsa_lookup_name_winbind(
 	NTSTATUS status;
 	const char *check_domain_name = NULL;
 	bool expect_domain = false;
+	struct imessaging_context *imsg_ctx =
+		dcesrv_imessaging_context(state->dce_call->conn);
 
 	if (item->name == NULL) {
 		/*
@@ -1991,9 +2070,9 @@ static NTSTATUS dcesrv_lsa_lookup_name_winbind(
 	}
 
 	state->wb.irpc_handle = irpc_binding_handle_by_name(state,
-					state->dce_call->msg_ctx,
-					"winbind_server",
-					&ndr_table_lsarpc);
+							    imsg_ctx,
+							    "winbind_server",
+							    &ndr_table_lsarpc);
 	if (state->wb.irpc_handle == NULL) {
 		DEBUG(0,("Failed to get binding_handle for winbind_server task\n"));
 		state->dce_call->fault_code = DCERPC_FAULT_CANT_PERFORM;
@@ -2018,6 +2097,8 @@ static NTSTATUS dcesrv_lsa_lookup_sid_winbind(
 	struct dom_sid domain_sid = {0,};
 	NTSTATUS status;
 	bool match;
+	struct imessaging_context *imsg_ctx =
+		dcesrv_imessaging_context(state->dce_call->conn);
 
 	/*
 	 * Verify the sid is not INVALID.
@@ -2108,9 +2189,9 @@ static NTSTATUS dcesrv_lsa_lookup_sid_winbind(
 	}
 
 	state->wb.irpc_handle = irpc_binding_handle_by_name(state,
-					state->dce_call->msg_ctx,
-					"winbind_server",
-					&ndr_table_lsarpc);
+							    imsg_ctx,
+							    "winbind_server",
+							    &ndr_table_lsarpc);
 	if (state->wb.irpc_handle == NULL) {
 		DEBUG(0,("Failed to get binding_handle for winbind_server task\n"));
 		state->dce_call->fault_code = DCERPC_FAULT_CANT_PERFORM;
