@@ -1,25 +1,26 @@
-/* 
+/*
    Unix SMB/CIFS implementation.
    SMB torture tester
    Copyright (C) Andrew Tridgell 1997-2003
    Copyright (C) Jelmer Vernooij 2006-2008
-   
+
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
-   
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include "includes.h"
-#include "lib/cmdline/popt_common.h"
+#include "lib/util/util_file.h"
+#include "lib/cmdline/cmdline.h"
 #include "system/time.h"
 #include "system/wait.h"
 #include "system/filesys.h"
@@ -72,7 +73,7 @@ static void print_test_list(const struct torture_suite *suite, const char *prefi
 }
 
 static bool run_matching(struct torture_context *torture,
-						 const char *prefix, 
+						 const char *prefix,
 						 const char *expr,
 						 const char **restricted,
 						 struct torture_suite *suite,
@@ -188,7 +189,7 @@ bool torture_parse_target(TALLOC_CTX *ctx,
 	NTSTATUS status;
 
 	/* see if its a RPC transport specifier */
-	if (!smbcli_parse_unc(target, NULL, &host, &share)) {
+	if (!smbcli_parse_unc(target, ctx, &host, &share)) {
 		const char *h;
 
 		status = dcerpc_parse_binding(ctx, target, &binding_struct);
@@ -210,6 +211,9 @@ bool torture_parse_target(TALLOC_CTX *ctx,
 		lpcfg_set_cmdline(lp_ctx, "torture:host", host);
 		lpcfg_set_cmdline(lp_ctx, "torture:share", share);
 		lpcfg_set_cmdline(lp_ctx, "torture:binding", host);
+
+		TALLOC_FREE(host);
+		TALLOC_FREE(share);
 	}
 
 	return true;
@@ -220,7 +224,7 @@ static void parse_dns(struct loadparm_context *lp_ctx, const char *dns)
 	char *userdn, *basedn, *secret;
 	char *p, *d;
 
-	/* retrievieng the userdn */
+	/* retrieve the userdn */
 	p = strchr_m(dns, '#');
 	if (!p) {
 		lpcfg_set_cmdline(lp_ctx, "torture:ldap_userdn", "");
@@ -419,6 +423,8 @@ int main(int argc, const char *argv[])
 	      OPT_DANGEROUS,OPT_SMB_PORTS,OPT_ASYNC,OPT_NUMPROGS,
 	      OPT_EXTRA_USER,};
 	TALLOC_CTX *mem_ctx = NULL;
+	struct loadparm_context *lp_ctx = NULL;
+	bool ok;
 
 	struct poptOption long_options[] = {
 		POPT_AUTOHELP
@@ -447,7 +453,7 @@ int main(int argc, const char *argv[])
 		 "run async tests", NULL},
 		{"num-async",    0, POPT_ARG_INT,  &torture_numasync,  0,
 		 "number of simultaneous async requests", NULL},
-		{"maximum-runtime", 0, POPT_ARG_INT, &max_runtime, 0, 
+		{"maximum-runtime", 0, POPT_ARG_INT, &max_runtime, 0,
 		 "set maximum time for smbtorture to live", "seconds"},
 		{"extra-user",   0, POPT_ARG_STRING, NULL, OPT_EXTRA_USER,
 		 "extra user credentials", NULL},
@@ -457,7 +463,8 @@ int main(int argc, const char *argv[])
 		POPT_COMMON_CONNECTION
 		POPT_COMMON_CREDENTIALS
 		POPT_COMMON_VERSION
-		{0}
+		POPT_LEGACY_S4
+		POPT_TABLEEND
 	};
 
 	setlinebuf(stdout);
@@ -473,36 +480,55 @@ int main(int argc, const char *argv[])
 	/* we are never interested in SIGPIPE */
 	BlockSignals(true, SIGPIPE);
 
-	pc = poptGetContext("smbtorture", argc, argv, long_options,
-			    POPT_CONTEXT_KEEP_FIRST);
+	ok = samba_cmdline_init(mem_ctx,
+				SAMBA_CMDLINE_CONFIG_CLIENT,
+				false /* require_smbconf */);
+	if (!ok) {
+		DBG_ERR("Unable to init cmdline parser\n");
+		TALLOC_FREE(mem_ctx);
+		exit(1);
+	}
+
+	pc = samba_popt_get_context(getprogname(),
+				    argc,
+				    argv,
+				    long_options,
+				    POPT_CONTEXT_KEEP_FIRST);
+	if (pc == NULL) {
+		DBG_ERR("Failed cmdline parser\n");
+		TALLOC_FREE(mem_ctx);
+		exit(1);
+	}
 
 	poptSetOtherOptionHelp(pc, "<binding>|<unc> TEST1 TEST2 ...");
 
+	lp_ctx = samba_cmdline_get_lp_ctx();
+
 	while((opt = poptGetNextOpt(pc)) != -1) {
 		switch (opt) {
 		case OPT_LOADFILE:
-			lpcfg_set_cmdline(cmdline_lp_ctx, "torture:loadfile", poptGetOptArg(pc));
+			lpcfg_set_cmdline(lp_ctx, "torture:loadfile", poptGetOptArg(pc));
 			break;
 		case OPT_UNCLIST:
-			lpcfg_set_cmdline(cmdline_lp_ctx, "torture:unclist", poptGetOptArg(pc));
+			lpcfg_set_cmdline(lp_ctx, "torture:unclist", poptGetOptArg(pc));
 			break;
 		case OPT_TIMELIMIT:
-			lpcfg_set_cmdline(cmdline_lp_ctx, "torture:timelimit", poptGetOptArg(pc));
+			lpcfg_set_cmdline(lp_ctx, "torture:timelimit", poptGetOptArg(pc));
 			break;
 		case OPT_NUMPROGS:
-			lpcfg_set_cmdline(cmdline_lp_ctx, "torture:nprocs", poptGetOptArg(pc));
+			lpcfg_set_cmdline(lp_ctx, "torture:nprocs", poptGetOptArg(pc));
 			break;
 		case OPT_DNS:
-			parse_dns(cmdline_lp_ctx, poptGetOptArg(pc));
+			parse_dns(lp_ctx, poptGetOptArg(pc));
 			break;
 		case OPT_DANGEROUS:
-			lpcfg_set_cmdline(cmdline_lp_ctx, "torture:dangerous", "Yes");
+			lpcfg_set_cmdline(lp_ctx, "torture:dangerous", "Yes");
 			break;
 		case OPT_ASYNC:
-			lpcfg_set_cmdline(cmdline_lp_ctx, "torture:async", "Yes");
+			lpcfg_set_cmdline(lp_ctx, "torture:async", "Yes");
 			break;
 		case OPT_SMB_PORTS:
-			lpcfg_set_cmdline(cmdline_lp_ctx, "smb ports", poptGetOptArg(pc));
+			lpcfg_set_cmdline(lp_ctx, "smb ports", poptGetOptArg(pc));
 			break;
 		case OPT_EXTRA_USER:
 			{
@@ -515,13 +541,15 @@ int main(int argc, const char *argv[])
 					talloc_free(mem_ctx);
 					exit(1);
 				}
-				lpcfg_set_cmdline(cmdline_lp_ctx, option, value);
+				lpcfg_set_cmdline(lp_ctx, option, value);
 				talloc_free(option);
 			}
 			break;
 		default:
 			if (opt < 0) {
-				printf("bad command line option %d\n", opt);
+				printf("Invalid command line option %s (%d)\n",
+				       poptBadOption(pc, 0),
+				       opt);
 				talloc_free(mem_ctx);
 				exit(1);
 			}
@@ -540,54 +568,54 @@ int main(int argc, const char *argv[])
 	}
 
 	if (strcmp(target, "samba3") == 0) {
-		lpcfg_set_cmdline(cmdline_lp_ctx, "torture:samba3", "true");
-		lpcfg_set_cmdline(cmdline_lp_ctx, "torture:resume_key_support", "false");
+		lpcfg_set_cmdline(lp_ctx, "torture:samba3", "true");
+		lpcfg_set_cmdline(lp_ctx, "torture:resume_key_support", "false");
 	} else if (strcmp(target, "samba4") == 0) {
-		lpcfg_set_cmdline(cmdline_lp_ctx, "torture:samba4", "true");
+		lpcfg_set_cmdline(lp_ctx, "torture:samba4", "true");
 	} else if (strcmp(target, "samba4-ntvfs") == 0) {
-		lpcfg_set_cmdline(cmdline_lp_ctx, "torture:samba4", "true");
-		lpcfg_set_cmdline(cmdline_lp_ctx, "torture:samba4-ntvfs", "true");
+		lpcfg_set_cmdline(lp_ctx, "torture:samba4", "true");
+		lpcfg_set_cmdline(lp_ctx, "torture:samba4-ntvfs", "true");
 	} else if (strcmp(target, "winxp") == 0) {
-		lpcfg_set_cmdline(cmdline_lp_ctx, "torture:winxp", "true");
+		lpcfg_set_cmdline(lp_ctx, "torture:winxp", "true");
 	} else if (strcmp(target, "w2k3") == 0) {
-		lpcfg_set_cmdline(cmdline_lp_ctx, "torture:w2k3", "true");
+		lpcfg_set_cmdline(lp_ctx, "torture:w2k3", "true");
 	} else if (strcmp(target, "w2k8") == 0) {
-		lpcfg_set_cmdline(cmdline_lp_ctx, "torture:w2k8", "true");
-		lpcfg_set_cmdline(cmdline_lp_ctx,
+		lpcfg_set_cmdline(lp_ctx, "torture:w2k8", "true");
+		lpcfg_set_cmdline(lp_ctx,
 		    "torture:invalid_lock_range_support", "false");
 	} else if (strcmp(target, "w2k12") == 0) {
-		lpcfg_set_cmdline(cmdline_lp_ctx, "torture:w2k12", "true");
+		lpcfg_set_cmdline(lp_ctx, "torture:w2k12", "true");
 	} else if (strcmp(target, "win7") == 0) {
-		lpcfg_set_cmdline(cmdline_lp_ctx, "torture:win7", "true");
-		lpcfg_set_cmdline(cmdline_lp_ctx, "torture:resume_key_support", "false");
-		lpcfg_set_cmdline(cmdline_lp_ctx, "torture:rewind_support", "false");
+		lpcfg_set_cmdline(lp_ctx, "torture:win7", "true");
+		lpcfg_set_cmdline(lp_ctx, "torture:resume_key_support", "false");
+		lpcfg_set_cmdline(lp_ctx, "torture:rewind_support", "false");
 
 		/* RAW-SEARCH for fails for inexplicable reasons against win7 */
-		lpcfg_set_cmdline(cmdline_lp_ctx, "torture:search_ea_support", "false");
+		lpcfg_set_cmdline(lp_ctx, "torture:search_ea_support", "false");
 
-		lpcfg_set_cmdline(cmdline_lp_ctx, "torture:hide_on_access_denied",
+		lpcfg_set_cmdline(lp_ctx, "torture:hide_on_access_denied",
 		    "true");
 	} else if (strcmp(target, "onefs") == 0) {
-		lpcfg_set_cmdline(cmdline_lp_ctx, "torture:onefs", "true");
-		lpcfg_set_cmdline(cmdline_lp_ctx, "torture:openx_deny_dos_support",
+		lpcfg_set_cmdline(lp_ctx, "torture:onefs", "true");
+		lpcfg_set_cmdline(lp_ctx, "torture:openx_deny_dos_support",
 		    "false");
-		lpcfg_set_cmdline(cmdline_lp_ctx, "torture:range_not_locked_on_file_close", "false");
-		lpcfg_set_cmdline(cmdline_lp_ctx, "torture:sacl_support", "false");
-		lpcfg_set_cmdline(cmdline_lp_ctx, "torture:ea_support", "false");
-		lpcfg_set_cmdline(cmdline_lp_ctx, "torture:smbexit_pdu_support",
+		lpcfg_set_cmdline(lp_ctx, "torture:range_not_locked_on_file_close", "false");
+		lpcfg_set_cmdline(lp_ctx, "torture:sacl_support", "false");
+		lpcfg_set_cmdline(lp_ctx, "torture:ea_support", "false");
+		lpcfg_set_cmdline(lp_ctx, "torture:smbexit_pdu_support",
 		    "false");
-		lpcfg_set_cmdline(cmdline_lp_ctx, "torture:smblock_pdu_support",
+		lpcfg_set_cmdline(lp_ctx, "torture:smblock_pdu_support",
 		    "false");
-		lpcfg_set_cmdline(cmdline_lp_ctx, "torture:2_step_break_to_none",
+		lpcfg_set_cmdline(lp_ctx, "torture:2_step_break_to_none",
 		    "true");
-		lpcfg_set_cmdline(cmdline_lp_ctx, "torture:deny_dos_support", "false");
-		lpcfg_set_cmdline(cmdline_lp_ctx, "torture:deny_fcb_support", "false");
-		lpcfg_set_cmdline(cmdline_lp_ctx, "torture:read_support", "false");
-		lpcfg_set_cmdline(cmdline_lp_ctx, "torture:writeclose_support", "false");
-		lpcfg_set_cmdline(cmdline_lp_ctx, "torture:resume_key_support", "false");
-		lpcfg_set_cmdline(cmdline_lp_ctx, "torture:rewind_support", "false");
-		lpcfg_set_cmdline(cmdline_lp_ctx, "torture:raw_search_search", "false");
-		lpcfg_set_cmdline(cmdline_lp_ctx, "torture:search_ea_size", "false");
+		lpcfg_set_cmdline(lp_ctx, "torture:deny_dos_support", "false");
+		lpcfg_set_cmdline(lp_ctx, "torture:deny_fcb_support", "false");
+		lpcfg_set_cmdline(lp_ctx, "torture:read_support", "false");
+		lpcfg_set_cmdline(lp_ctx, "torture:writeclose_support", "false");
+		lpcfg_set_cmdline(lp_ctx, "torture:resume_key_support", "false");
+		lpcfg_set_cmdline(lp_ctx, "torture:rewind_support", "false");
+		lpcfg_set_cmdline(lp_ctx, "torture:raw_search_search", "false");
+		lpcfg_set_cmdline(lp_ctx, "torture:search_ea_size", "false");
 	}
 
 	if (max_runtime) {
@@ -603,16 +631,16 @@ int main(int argc, const char *argv[])
 	if (extra_module != NULL) {
 		init_module_fn fn = load_module(poptGetOptArg(pc), false, NULL);
 
-		if (fn == NULL) 
+		if (fn == NULL)
 			d_printf("Unable to load module from %s\n", poptGetOptArg(pc));
 		else {
 			status = fn(mem_ctx);
 			if (NT_STATUS_IS_ERR(status)) {
-				d_printf("Error initializing module %s: %s\n", 
+				d_printf("Error initializing module %s: %s\n",
 					poptGetOptArg(pc), nt_errstr(status));
 			}
 		}
-	} else { 
+	} else {
 		torture_init(mem_ctx);
 	}
 
@@ -620,7 +648,6 @@ int main(int argc, const char *argv[])
 		print_testsuite_list();
 		poptFreeContext(pc);
 		talloc_free(mem_ctx);
-		popt_free_cmdline_credentials();
 		return 0;
 	}
 
@@ -644,13 +671,12 @@ int main(int argc, const char *argv[])
 		}
 		poptFreeContext(pc);
 		talloc_free(mem_ctx);
-		popt_free_cmdline_credentials();
 		return 0;
 	}
 
 	if (torture_seed == 0) {
 		torture_seed = time(NULL);
-	} 
+	}
 	printf("Using seed %d\n", torture_seed);
 	srandom(torture_seed);
 
@@ -700,7 +726,7 @@ int main(int argc, const char *argv[])
 		return 1;
 	}
 
-	torture->lp_ctx = cmdline_lp_ctx;
+	torture->lp_ctx = lp_ctx;
 
 	gensec_init();
 
@@ -711,7 +737,7 @@ int main(int argc, const char *argv[])
 
 		/* At this point, we should just have a target string,
 		 * followed by a series of test names. Unless we are in
-		 * shell mode, in which case we don't need anythig more.
+		 * shell mode, in which case we don't need anything more.
 		 */
 
 		if (argc_new < 3) {
@@ -719,7 +745,7 @@ int main(int argc, const char *argv[])
 			usage(pc);
 			torture->results->returncode = 1;
 		} else if (!torture_parse_target(torture,
-					cmdline_lp_ctx, argv_new[1])) {
+					lp_ctx, argv_new[1])) {
 			/* Take the target name or binding. */
 			usage(pc);
 			torture->results->returncode = 1;
@@ -739,7 +765,6 @@ int main(int argc, const char *argv[])
 	if (torture->results->returncode && correct) {
 		poptFreeContext(pc);
 		talloc_free(mem_ctx);
-		popt_free_cmdline_credentials();
 		return(0);
 	} else {
 		poptFreeContext(pc);
